@@ -1,7 +1,7 @@
 use v6;
 use fatal;
 
-unit class Getopt::Long:ver<0.1.1>;
+unit class Getopt::Long:ver<0.1.2>;
 
 my sub null-converter(Str:D $value --> Str) {
 	return $value;
@@ -23,6 +23,10 @@ my sub real-converter(Str:D $value --> Real) {
 	return $value.Real;
 }
 
+my sub complex-converter(Str:D $value --> Complex) {
+	return $value.Complex;
+}
+
 my sub maybe-converter(Str:D $value --> Any) {
 	return val($value);
 }
@@ -64,9 +68,7 @@ my class HashStore does Store {
 		$hash{$!key}{$key} = $!converter($value);
 	}
 	method store-direct(Any:D $pair, Hash:D $hash) {
-		my ($key, $value) = $pair.split('=', 2);
-		$hash{$!key} //= $!type === Any ?? Hash !! Hash[$!type].new;
-		$hash{$!key}{$key} = $value;
+		!!!
 	}
 }
 
@@ -97,7 +99,7 @@ my %store-for = (
 	''  => ScalarStore,
 );
 
-my sub make-option(@names, $multi-class, $multi-args, $arity, %options-args, $negatable) {
+my sub make-option(@names, $multi-class, $multi-args, $arity, %options-args?, $negatable?) {
 	return flat @names.map: -> $name {
 		my $store = $multi-class.new(|%$multi-args, :key(@names[0]));
 		my @options;
@@ -114,8 +116,7 @@ my grammar Argument {
 	token TOP {
 		<names> <argument>
 		{
-			my ($multi-class, $multi-args, $arity, $options-args, $negatable) = $<argument>.ast;
-			make make-option($<names>.ast, $multi-class, $multi-args, $arity, $options-args, $negatable);
+			make make-option($<names>.ast, |$<argument>.ast);
 		}
 	}
 
@@ -131,29 +132,31 @@ my grammar Argument {
 
 	token boolean {
 		$<negatable>=['!'?]
-		{ make [ ScalarStore, {}, 0..0, {}, $<negatable> ] }
+		{ make [ ScalarStore, {}, 0..0, {}, ?$<negatable> ] }
 	}
 
 	token counter {
 		'+'
-		{ make [ CountStore, {}, 0..0, { }, False ] }
+		{ make [ CountStore, {}, 0..0 ] }
 	}
 
 	my %converter-for-format = (
 		i => &int-converter,
+		o => &int-converter, # compatability with p5
 		s => &null-converter,
-		f => &real-converter,
+		f => &num-converter,
 		r => &rat-converter,
+		c => &complex-converter,
 	);
 
 	token type {
-		<[sifor]>
+		<[siforc]>
 		{ make %converter-for-format{$/} }
 	}
 
 	token equals {
 		'=' <type> $<repeat>=[<[%@]>?]
-		{ make [ %store-for{~$<repeat>}, { :converter($<type>.ast) }, 1..1, { }, False ] }
+		{ make [ %store-for{~$<repeat>}, { :converter($<type>.ast) }, 1..1 ] }
 	}
 
 	rule range {
@@ -162,22 +165,22 @@ my grammar Argument {
 	}
 	token equals-more {
 		'=' <type> '{' <range>'}'
-		{ make [ ArrayStore, { :converter($<type>.ast) }, $<range>.ast, { }, False ] }
+		{ make [ ArrayStore, { :converter($<type>.ast) }, $<range>.ast ] }
 	}
 
 	token colon-type {
 		':' <type>
-		{ make [ ScalarStore, { :converter($<type>.ast) }, 0..1, { :default($<type>.ast.returns.new) }, False ] }
+		{ make [ ScalarStore, { :converter($<type>.ast) }, 0..1, { :default($<type>.ast.returns.new) } ] }
 	}
 
 	token colon-int {
 		':' $<num>=[<[0..9]>+]
-		{ make [ ScalarStore, { :converter(&int-converter) }, 0..1, { :default($<num>.Int) }, False ] }
+		{ make [ ScalarStore, { :converter(&int-converter) }, 0..1, { :default($<num>.Int) } ] }
 	}
 
 	token colon-count {
 		':+'
-		{ make [ CountStore, { :converter(&int-converter) }, 0..1, { :default(1) }, False ] }
+		{ make [ CountStore, { :converter(&int-converter) }, 0..1, { :default(1) } ] }
 	}
 }
 
@@ -197,12 +200,13 @@ method new-from-patterns(@patterns, *%args) {
 }
 
 my %converter-for-type{Any:U} = (
-	(Int)  => &int-converter,
-	(Rat)  => &rat-converter,
-	(Num)  => &num-converter,
-	(Real) => &real-converter,
-	(Str)  => &null-converter,
-	(Any)  => &maybe-converter,
+	(Int)     => &int-converter,
+	(Rat)     => &rat-converter,
+	(Num)     => &num-converter,
+	(Real)    => &real-converter,
+	(Complex) => &complex-converter,
+	(Str)     => &null-converter,
+	(Any)     => &maybe-converter,
 );
 
 my role Formatted {
@@ -225,36 +229,20 @@ my sub parse-parameter(Parameter $param) {
 		}
 	}
 	else {
-		my $key = @names[0];
 		if $param.sigil eq '$' {
 			my $type = $param.type;
 			if $param.type === Bool {
-				my $store = ScalarStore.new(:$key);
-				return flat @names.map: -> $name {
-					my @options;
-					@options.push: Option.new(:$name, :$store, :arity(0..0), :default);
-					if $param.default {
-						@options.push: Option.new(:name("no$name") , :$store, :arity(0..0), :!default);
-						@options.push: Option.new(:name("no-$name"), :$store, :arity(0..0), :!default);
-					}
-					@options;
-				}
+				return make-option(@names, ScalarStore, {}, 0..0, {}, $param.default)
 			}
 			else {
 				my $converter = %converter-for-type{$param.type} // &null-converter;
-				my $store = ScalarStore.new(:$key, :$converter);
-				return @names.map: -> $name {
-					Option.new(:$name, :$store, :arity(1..1));
-				}
+				return make-option(@names, ScalarStore, { :$converter }, 1..1);
 			}
 		}
 		else {
 			my $type = $param.type.of ~~ Any ?? $param.type.of !! Any;
 			my $converter = %converter-for-type{$type} // &null-converter;
-			my $store = %store-for{$param.sigil}.new(:$key, :$type, :$converter);
-			return @names.map: -> $name {
-				Option.new(:$name, :$store, :arity(1..1));
-			}
+			return make-option(@names, %store-for{$param.sigil}, { :$type, :$converter }, 1..1);
 		}
 	}
 }
@@ -297,6 +285,10 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 
 		my $consumed = 0;
 
+		sub get-option($key) {
+			return %!options{$key} // die "Unknown option $key";
+		}
+
 		sub take-value($option, $value) {
 			$option.store($value, %hash);
 			$consumed++;
@@ -305,7 +297,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 				when X::Str::Numeric {
 					$_ does role {
 						method message() {
-							"Cannot convert --{$option.name} argument to number: $.reason $.source-indicator";
+							qq{Cannot convert --$option.name() argument "$value" to number: $.reason};
 						}
 					}
 					.rethrow;
@@ -341,7 +333,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 			my @values = $<values>.Str.comb;
 			for @values.keys -> $index {
 				my $value = @values[$index];
-				my $option = %!options{$value} or die "No such option $value";
+				my $option = get-option($value);
 				if $option.arity.max > 0 && $index + 1 < @values.elems {
 					take-value($option, $<values>.substr($index + 1));
 				}
@@ -355,23 +347,13 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 			last;
 		}
 		elsif $head ~~ / ^ '-' ** 1..2 $<name>=[\w <[\w-]>*] $ / -> $/ {
-			if %!options{$<name>} -> $option {
-				take-args($option);
-			}
-			else {
-				die "Unknown option $<name>";
-			}
+			take-args(get-option($<name>));
 		}
 		elsif $head ~~ / ^ '--' $<name>=[<[\w-]>+] '=' $<value>=[.*] / -> $/ {
-			if %!options{$<name>} -> $option {
-				die  "$<name> doesn't take arguments" if $option.arity.max == 0;
-				take-value($option, ~$<value>);
-
-				take-args($option);
-			}
-			else {
-				die "Unknown option $<name>";
-			}
+			my $option = get-option($<name>);
+			die  "$<name> doesn't take arguments" if $option.arity.max == 0;
+			take-value($option, ~$<value>);
+			take-args($option);
 		}
 		else {
 			if $permute {
@@ -808,8 +790,11 @@ argument to start with C<-> or C<-->.
 =begin item2
 i
 
-Integer. An optional leading plus or minus sign, followed by a
-sequence of digits.
+Integer. This can be either an optional leading plus or minus sign,
+followed by a sequence of digits, or an octal string (C<0o>, optionally
+followed by '0', '1', .. '7'), or a hexadecimal string (C<0x> followed
+by '0' .. '9', 'a' .. 'f', case insensitive), or a binary string (C<0b>
+followed by a series of '0' and '1').
 
 =end item2
 
@@ -823,7 +808,14 @@ Rational number. For example C<3.14>.
 =begin item2
 f
 
-Real number. For example C<3.14>, C<-6.23E24> and so on.
+Floating-pointer number. For example C<3.14>, C<-6.23E24> and so on.
+
+=end item2
+
+=begin item2
+c
+
+Complex number. For example C<1+2i>.
 
 =end item2
 
